#!/usr/bin/env python3
import npyscreen
import configparser
import subprocess
import io
import os
import gettext
import sys

from xdg.BaseDirectory import xdg_config_home

_ = gettext.gettext

class App(npyscreen.StandardApp):
    def onStart(self):
        self.addForm('MAIN', MainForm, name=_('Integrity Verification'))

def output_event_handler(self, event):
        self.myFixedText.value = int(self.myFixedText.value) + 1
        self.myFixedText.display()

class FiltersEdit(npyscreen.BoxTitle):
    _contained_widget = npyscreen.MultiLineEdit

class StatusText(npyscreen.BoxTitle):
    _contained_widget = npyscreen.FixedText

class MainForm(npyscreen.ActionForm):
    def create(self):
        self.config = configparser.ConfigParser(allow_no_value=True)
        self.config.optionxform = lambda option: option
        self.config.read(xdg_config_home + '/rusbitech/fly-admin-int-check.conf')

        with open ("/etc/astra_version", "r") as versionFile:
            self.ce_edition = (versionFile.read(2) == 'CE')

        y, x = self.useable_space()
        self.mount_point = self.add(npyscreen.TitleFilenameCombo,
                                    name=_('Media mount point:'),
                                    value=self.config.get('MainWindow', 'mediaMountPoint', fallback='/media/cdrom'),
                                    begin_entry_at=25,
                                    select_dir=True,
                                    must_exists=True,
                                    hidden = self.ce_edition,
                                    editable = not self.ce_edition)

        self.md5_checkbox = self.add(npyscreen.CheckBox, name=_('Use MD5 sums of installed packages'),
                                     value=self.config.getboolean('MainWindow', 'useMd5Sums', fallback=self.ce_edition),
                                     hidden = self.ce_edition)

        if self.ce_edition:
            self.md5_checkbox.value = True

        force_filters = ['/boot/*',
                         '/bin/*',
                         '/lib/*',
                         '/usr/bin/*',
                         '/lib/security/*',
                         '/etc/init.d/*'];
        ignore_filters = ['/etc/*',
                          '/var/*',
                          '/tmp/*',
                          '/proc/*',
                          '/sys/*',
                          '/usr/share/pam-configs/*',
                         '/usr/lib/libreoffice/share/registry/main.xcd',
                         '/usr/lib/libreoffice/share/config/soffice.cfg/modules/*']

        force_filters = self.config.get('MainWindow', 'forceFilters', fallback=', '.join(force_filters)).split(', ')
        ignore_filters = self.config.get('MainWindow', 'ignoreFilters', fallback=', '.join(ignore_filters)).split(', ')

        self.force_filters_edit = self.add(FiltersEdit, name=_('Force filters'), value='\n'.join(force_filters), max_height=8, max_width=76)
        self.ignore_filters_edit = self.add(FiltersEdit, name=_('Ignore filters'), value='\n'.join(ignore_filters), max_height=8, max_width=76)

        self.txt_checkbox = self.add(npyscreen.CheckBox, name=_('Enabled plain text report'),
                                     value=self.config.getboolean('MainWindow', 'generateTxtReport', fallback=False))
        self.html_checkbox = self.add(npyscreen.CheckBox, name=_('Enable HTML report'),
                                      value=self.config.getboolean('MainWindow', 'generateHtmlReport', fallback=False))
        self.xml_checkbox = self.add(npyscreen.CheckBox, name=_('Enable XML report'),
                                     value=self.config.getboolean('MainWindow', 'generateXmlReport', fallback=False))

        self.txt_report_edit = self.add(npyscreen.TitleFilenameCombo, name=_('Plain text report file:'),
                                        value=self.config.get('MainWindow', 'txtReportFile', fallback=os.getenv('HOME')+'/report.txt'),
                                        begin_entry_at=30)
        self.html_report_edit = self.add(npyscreen.TitleFilenameCombo, name=_('HTML report file:'),
                                         value=self.config.get('MainWindow', 'htmlReportFile', fallback=os.getenv('HOME')+'/report.html'),
                                         begin_entry_at=30)
        self.xml_report_edit = self.add(npyscreen.TitleFilenameCombo, name=_('XML report file:'),
                                        value=self.config.get('MainWindow', 'xmlReportFile', fallback=os.getenv('HOME')+'/report.xml'),
                                        begin_entry_at=30)
        self.status_text = self.add(StatusText, name=_("Progress"), max_height=3, max_width=76)

    def while_waiting(self):
        if hasattr(self, 'p'):
            if self.p.poll() is None:
                self.status_text.value = self.p.stdout.readline()
                self.status_text.display()
            else:
                self.status_text.value = _('Finished')
                self.status_text.display()

    def on_ok(self):
        #write config
        if not self.config.has_section('MainWindow'):
            self.config.add_section('MainWindow')
        self.config.set('MainWindow', 'mediaMountPoint', self.mount_point.value)
        self.config.set('MainWindow', 'useMd5Sums', str(self.md5_checkbox.value))
        self.config.set('MainWindow', 'forceFilters', ', '.join(self.force_filters_edit.value.split('\n')))
        self.config.set('MainWindow', 'ignoreFilters', ', '.join(self.ignore_filters_edit.value.split('\n')))
        self.config.set('MainWindow', 'generateTxtReport', str(self.txt_checkbox.value))
        self.config.set('MainWindow', 'generateHtmlReport', str(self.html_checkbox.value))
        self.config.set('MainWindow', 'generateXmlReport', str(self.xml_checkbox.value))
        self.config.set('MainWindow', 'txtReportFile', self.txt_report_edit.value)
        self.config.set('MainWindow', 'htmlReportFile', self.html_report_edit.value)
        self.config.set('MainWindow', 'xmlReportFile', self.xml_report_edit.value)
        with open(xdg_config_home + '/rusbitech/fly-admin-int-check.conf', 'w') as configfile:
            self.config.write(configfile)

        #build args
        #args = ['/home/user/projects/fly-admin-int-check/astra-int-check/debug/astra-int-check']
        args = ['astra-int-check']

        if not self.md5_checkbox.value:
           args.extend(['-p', self.mount_point.value])
        if self.force_filters_edit.value:
            args.extend(['--force-filters', ','.join(self.force_filters_edit.value.split('\n'))])
        if self.ignore_filters_edit.value:
            args.extend(['--ignore-filters', ','.join(self.ignore_filters_edit.value.split('\n'))])
        if self.txt_checkbox.value:
            args.extend(['--txt', self.txt_report_edit.value])
        if self.html_checkbox.value:
            args.extend(['--html', self.html_report_edit.value])
        if self.xml_checkbox.value:
            args.extend(['--xml', self.xml_report_edit.value])

        self.p = subprocess.Popen(args, stdout=subprocess.PIPE, stderr=subprocess.DEVNULL, universal_newlines=True)

    def on_cancel(self):
        self.parentApp.switchForm(None)

if __name__ == '__main__':
    gettext.bindtextdomain(os.path.basename(sys.argv[0]))
    gettext.textdomain(os.path.basename(sys.argv[0]))
    MyApp = App()
    MyApp.run()
